<?php

namespace Imagine;

class Colors extends \ArrayObject {
    use arrayobject;

    protected $image;

    public static function make($image = null) {
        return new static($image);
    }

    public function __construct($image) {
        parent::__construct();
        
        $this->image = $image;
    }

    public static function convertHexToRGB($hex) {
        return [
            'r' => (int) base_convert(substr($hex, 0, 2), 16, 10),
            'g' => (int) base_convert(substr($hex, 2, 2), 16, 10),
            'b' => (int) base_convert(substr($hex, 4, 2), 16, 10),
        ];
    }

    function compare($c1, $c2, $tolerance = 35) {
        $c1 = is_array($c1) ? $c1 : [
            "r" => hexdec(substr($c1, 0, 2)),
            "g" => hexdec(substr($c1, 2, 2)),
            "b" => hexdec(substr($c1, 4, 2))
        ];
        $c2 = is_array($c2) ? $c2 : [
            "r" => hexdec(substr($c2, 0, 2)),
            "g" => hexdec(substr($c2, 2, 2)),
            "b" => hexdec(substr($c2, 4, 2))
        ];

        return ($c1['r'] >= $c2['r'] - $tolerance && $c1['r'] <= $c2['r'] + $tolerance) &&
               ($c1['g'] >= $c2['g'] - $tolerance && $c1['g'] <= $c2['g'] + $tolerance) &&
               ($c1['b'] >= $c2['b'] - $tolerance && $c1['b'] <= $c2['b'] + $tolerance);
    }

    /*  *
     *  @param mixed $color  :
            array = [ int r, int g, int b, int a ],
            string = "hex:alpha" ( ex: "1bd4a9:94" )
     *
    */
    public static function allocate(&$img, $color = null) {

        if ( $color !== null ) {
            # If we've received an Hex color , we must convert it into rgb(a).
            if ( is_string($color) ) {
                $hex = explode(':', $color);

                $color = self::convertHexToRGB($hex[0]);
                $color['a'] = isset($hex[1]) ? (float)$hex[1] : null;
            }

            $colors['a'] = isset($colors['a']) ? $colors['a'] : null;

            return $color['a'] !== null ? \imagecolorallocatealpha($img, $color['r'], $color['g'], $color['b'], (1 - $color['a']) * 127) : \imagecolorallocate($img, $color['r'], $color['g'], $color['b']);
        }

        return null;
    }

    public function offsetSet($offset, $value) {
        if ( is_null($offset) ) {
            $this->_container[] = self::allocate($this->image, $value);
        } else {
            if (is_string($value)) {
                $this->_container[$offset] = self::allocate($this->image, $value);
            }
            else {
                $this->_container[$offset] = $value;
            }
        }
    }


}
