<?php

namespace Imagine;

class Merge {

    public static function simple(resource &$destImg, resource $srcImg, int $destPosX = 0, int $destPosY = 0, int $srcPosX = 0, int $srcPosY = 0) : bool
    {
        return \imagecopy($destImg, $srcImg, $destPosX, $destPosY, $srcPosX, $srcPosY, \imageSX($srcImg), \imageSY($srcImg));
    }

    public static function alpha(resource &$destImg, resource &$srcImg, int $destX, itn $destY, int $srcX, int $srcY, int $srcW, int $srcH, int $percentage = 0) : bool
    {
        $destW = \imagesx($destImg);
        $destH = \imagesy($destImg);

        for ($y = 0; $y < $srcH + $srcY; $y++) {
            for ($x = 0; $x < $srcW + $srcX; $x++) {
                if ($x + $destX >= 0 && $x + $destX < $destW && $x + $srcX >= 0 && $x + $srcX < $srcW && $y + $destY >= 0 && $y + $destY < $destH && $y + $srcY >= 0 && $y + $srcY < $srcH) {

                    $destPixel = \imagecolorsforindex($destImg, \imagecolorat($destImg, $x + $destX, $y + $destY));
                    $srcImgColorat = \imagecolorat($srcImg, $x + $srcX, $y + $srcY);

                    if ($srcImgColorat >= 0) {
                        $srcPixel = \imagecolorsforindex($srcImg, $srcImgColorat);

                        $srcAlpha = 1 - ($srcPixel['alpha'] / 127);
                        $destAlpha = 1 - ($destPixel['alpha'] / 127);
                        $opacity = $srcAlpha * $pct / 100;

                        if ($destAlpha >= $opacity) {
                            $alpha = $destAlpha;
                        }

                        if ($destAlpha < $opacity) {
                            $alpha = $opacity;
                        }

                        if ($alpha > 1) {
                            $alpha = 1;
                        }

                        if ($opacity > 0) {
                            $destRed = round((($destPixel['red'] * $destAlpha * (1 - $opacity))));
                            $destGreen = round((($destPixel['green'] * $destAlpha * (1 - $opacity))));
                            $destBlue = round((($destPixel['blue'] * $destAlpha * (1 - $opacity))));
                            $srcRed = round((($srcPixel['red'] * $opacity)));
                            $srcGreen = round((($srcPixel['green'] * $opacity)));
                            $srcBlue = round((($srcPixel['blue'] * $opacity)));
                            $red = round(($destRed + $srcRed  ) / ($destAlpha * (1 - $opacity) + $opacity));
                            $green = round(($destGreen + $srcGreen) / ($destAlpha * (1 - $opacity) + $opacity));
                            $blue = round(($destBlue + $srcBlue ) / ($destAlpha * (1 - $opacity) + $opacity));

                            if ($red > 255) {
                                $red = 255;
                            }

                            if ($green > 255) {
                                $green = 255;
                            }

                            if ($blue > 255) {
                                $blue = 255;
                            }

                            $alpha = round((1 - $alpha) * 127);
                            $color = \imagecolorallocatealpha($destImg, $red, $green, $blue, $alpha);
                            \imagesetpixel($destImg, $x + $destX, $y + $destY, $color);
                        }
                    }
                }
            }
        }

        return true;
    }
}
