<?php

namespace Imagine;

trait arrayobject {

    protected $pointer   = null;
    protected $container = [];
    protected $changed   = [];
    protected $selected  = false;

    public function count() {
        return count( $this->container() );
    }

    public function contains($term, $strict = false) {
        return (array_search($term, $this->container(), $strict) !== false) ;
    }

    public function &arrayobjectCurrent() {
        if ( !is_null($this->pointer) ) {
            $var = &$this->container()[$this->pointer] ?: [];
            $var || ( $var = [] );
            return $var;
        }

        if ( $this->selected !== false ){
            $ret = &$this->selected ?: [];
            return $ret;
        }

        # Restoring integrity of container since it could be nullified
        is_array($this->container()) || $this->container([]);

        return $this->container();
    }

    public function offsetSet($offset, $value, $changed = null) {
        if ( $changed && (!isset($this->arrayobjectCurrent()[$offset]) || ($this->arrayobjectCurrent()[$offset] !== $value) ) ) {
            $this->changed($offset, true);
        }

        return is_null($offset) ? $this->arrayobject_current()[] = $value : $this->arrayobject_current()[$offset] = $value;
    }

    public function arrayobjectSetPointer($pointer) {
        # $pointer could nullify obj pointer
        if ( $this->pointer = $pointer ) {
            # Creating dataset whenever we have a new one
            if ( !isset($this->container()[$this->pointer]) ) {
                $this->container()[$this->pointer] = [];
                $this->changed[$this->pointer]   = [];
            }
        }

        return $this;
    }

    public function arrayobjectSelect($selection, $purge = true) {
        if ( is_bool($selection) ) {
            return $this->selected = $selection;
        }

        $purge && ( $this->selected = [] );

        foreach($selection as $pointer) {
            $this->selected[$pointer] = &$this->container[$pointer];
        }

        return true;
    }

    public function arrayobjectExist($pointer) {
        return isset( $this->container()[$pointer] );
    }

    public function arrayobjectFlushChanged() {
        ! is_null($this->pointer) ?
            $this->changed[$this->pointer] = []
            :
            $this->changed = []
        ;
    }

    public function changed($offset = null, $set = null) {
        if ($offset) {
            if ($set !== null) {
                ! is_null($this->pointer) ?
                    ( $this->changed[$this->pointer][$offset] = $set )
                    :
                    ( $this->changed[$offset] = $set );
            }

            return !is_null($this->pointer) ? $this->changed[$this->pointer][$offset] : $this->changed[$offset];
        }

        return array_keys( !is_null($this->pointer)
                ?   $this->changed[$this->pointer] ?? []
                :   $this->changed) ?? [];
    }

    public function arrayobjectRemove($pointer) {
        if ( isset($this->container()[$pointer]) ) {
            unset( $this->container()[$pointer], $this->changed[$pointer]);
        }
    }

    public function arrayobjectIterate($callback) {
        if ( $callback && is_callable($callback) ) {
            $pointer = $this->pointer;

            foreach($this->container() as $key => $value) {
                $this->arrayobjectSetPointer($key);
                $callback($key, $this);
            }

            $this->arrayobjectSetPointer($pointer);
        }

        return $this;
    }

    public function offsetGet($offset) {
        if ( !is_null($this->pointer) ) {
            return isset($this->container()[$this->pointer][$offset]) ? $this->container()[$this->pointer][$offset] : null;
        }
        else {
            return isset($this->container()[$offset]) ? $this->container()[$offset] : null;
        }
    }

    public function offsetExists($offset) {
        return array_key_exists($offset, $this->arrayobjectCurrent() );
    }

    public function offsetUnset($offset) {
        if ( !is_null($this->pointer)) {
            unset($this->container()[$this->pointer][$offset]);
        }
        else {
            unset($this->container()[$offset]) ;
        }
    }

    public function sort($field, $order = 'ASC') {
        # Arrayobj::order_by($this->container(), $field);
        # $order === 'DESC' && array_reverse($this->arrayobject_current());
    }

    public function rewind()  {
        reset( $this->container() );

        # Rewinding will also reset the pointer
        $this->arrayobjectSetPointer(key($this->container()));

        return $this;
    }

    public function current() {
        return $this->arrayobjectSetPointer( $this->key() );
    }

    public function key() {
        $var = key( $this->container() );
        return $var;
    }

    public function next() {
        $var = next( $this->container() );
        return $var;
    }

    public function valid() {
        $key = $this->key();
        return ( $key !== NULL ) && ( $key !== FALSE );
    }

    protected function &container($set = null) {
        if ( $set !== null ) {
            $this->container = $set;
        }

        if (  $this->selected !== false ) {
            return $this->selected;
        }

        return $this->container;
    }
}
